<?php

/**
 * Memcache CacheResource
 *
 * CacheResource Implementation based on the KeyValueStore API to use
 * memcache as the storage resource for Smarty's output caching.
 *
 * Note that memcache has a limitation of 256 characters per cache-key.
 * To avoid complications all cache-keys are translated to a sha1 hash.
 *
 * @package CacheResource-examples
 * @author Rodney Rehm
 */

include_once LAB_PATH_KERNEL.'/classes/Smarty/sysplugins/smarty_cacheresource_keyvaluestore.php';
include_once LAB_PATH_KERNEL.'/classes/Cache/Generic.php';
include_once LAB_PATH_KERNEL.'/classes/Cache/Memcache.php';

class Smarty_CacheResource_Memcache extends Smarty_CacheResource_KeyValueStore {
	/**
	 * memcache instance
	 * @var _cacher
	 */
	protected $_cacher = null;
	
	public function __construct()
	{
		$this->_cacher = new Cache_Generic('memcache');
	}
	
	/**
	 * Read values for a set of keys from cache
	 *
	 * @param array $keys list of keys to fetch
	 * @return array list of values with the given keys used as indexes
	 * @return boolean true on success, false on failure
	 */
	protected function read(array $keys)
	{
		$_keys = $lookup = array();
		foreach ($keys as $k) {
			$_k = sha1($k);
			$_keys[] = $_k;
			$lookup[$_k] = $k;
		}
		$_res = array();
		if (!$res = $this->_cacher->get($_keys)) return $_res;
		foreach ($res as $k => $v) {
			$_res[$lookup[$k]] = $v;
		}
		return $_res;
	}
	
	/**
	 * Save values for a set of keys to cache
	 *
	 * @param array $keys list of values to save
	 * @param int $expire expiration time
	 * @return boolean true on success, false on failure
	 */
	protected function write(array $keys, $expire=null)
	{
		foreach ($keys as $k => $v) {
			$k = sha1($k);
			$this->_cacher->set($k, $v, $expire);
		}
		return true;
	}

	/**
	 * Remove values from cache
	 *
	 * @param array $keys list of keys to delete
	 * @return boolean true on success, false on failure
	 */
	protected function delete(array $keys)
	{
		foreach ($keys as $k) {
			$k = sha1($k);
			$this->_cacher->remove($k);
		}
		return true;
	}

	/**
	 * Remove *all* values from cache
	 *
	 * @return boolean true on success, false on failure
	 */
	protected function purge()
	{
		return $this->_cacher->clean();
	}
}
